# ==============================================================================
testNegativeBinomialModel <- function
( Model,
  SupportIndex=NULL,
  Nsim=10000
){

  require(MASS)

  if (class(Model)[1] != "negbin"){
      stop("The model should be of class negbin.")
  }

  Fitted <- fitted(Model)

  Counts <- Model$y

  K <- max(Counts, na.rm=TRUE) + 1
  Expected <- Observed <- numeric(K)
  for(k in 0:K){
    Expected[k] <- sum( dpois(x=k-1, lambda=Fitted) )
    Observed[k] <- sum( Counts==k-1 )
  }

  if (!is.null(SupportIndex)){
    Expected <- Expected[SupportIndex]
    Observed <- Observed[SupportIndex]
  }

  Pvalue <- SimPvalues (Expected, Observed, Nsim=Nsim)

  testDiscreteDistributionObj <- list(Pvalue=Pvalue,
    Observed=Observed,
    Expected=Expected,
    Model=Model
  )

  class(testDiscreteDistributionObj) <- "testDiscreteDistribution"

  return(testDiscreteDistributionObj)
}
# ==============================================================================
