# ==============================================================================
testPoissonModel <- function
(
  Model,
  SupportIndex = NULL,
  Nsim = 10000
){

  if (class(Model)[1] == "glmerMod"){
    Counts <- Model@resp$y
  } else if (class(Model)[1] == "glm" | class(Model)[1] == "lm"){
    Counts <- Model$y
  } else stop("Model should be of class glmerMod, glm or lm.")

  Fitted <- fitted(Model)

  K <- max(Counts, na.rm=TRUE) + 1
  Expected <- Observed <- numeric(K)
  for(k in 0:K){
    Expected[k] <- sum( dpois(x=k-1, lambda=Fitted) )
    Observed[k] <- sum( Counts==k-1 )
  }

  if (!is.null(SupportIndex)){
    Expected <- Expected[SupportIndex]
    Observed <- Observed[SupportIndex]
  }

  Pvalue <- SimPvalues (Expected, Observed, Nsim=Nsim)

  testDiscreteDistributionObj <- list(Pvalue=Pvalue,
    Observed=Observed,
    Expected=Expected,
    Model=Model,
    SupportIndex = SupportIndex
  )

  class(testDiscreteDistributionObj) <- "testDiscreteDistribution"

  return(testDiscreteDistributionObj)
}
# ==============================================================================

# ==============================================================================
qqpois <- function
( x,
  lambda = mean(x),
  main = " ",
  sub = " ",
  qqline = TRUE,
  qqlineAll = TRUE,
  Local = FALSE,
  cex = 1.5,
  pch = 19,
  testAdherence = FALSE,
  Nsim = 9999,
  digits = 3
){

  if(testAdherence){
    Pvalue <- round(testPoissonDistribution(x, Nsim = Nsim), digits = digits)
    sub <- paste("P-value for adherence to a Poisson distribution =", Pvalue)
  }

  ox <- x[order(x)]
  emp <- ecdf(ox)(ox)
  teor <- ppois(ox, lambda=lambda)
  if(Local){
    plot(teor,emp, xlab="Poisson Theoretical Quantiles", ylab="Sample Quantiles",
      main=main, sub=sub, ylim=c(min(emp),max(emp)), xlim=c(min(teor),max(teor)), cex=cex, pch=pch, col='blue')
  }
  else{
    plot(teor,emp, xlab="Poisson Theoretical Quantiles", ylab="Sample Quantiles",
      main=main, sub=sub ,ylim=c(0,1), xlim=c(0,1), cex=cex, pch=pch, col='blue')
  }
  if(qqline){
    lines(c(min(emp),max(emp)), c(min(emp),max(emp)), col="black", lwd=2)
  }
  if(qqlineAll){
    abline(0,1, col="red")
  }

}
# ==============================================================================

# ==============================================================================
hpois <- function
( x,
  lambda = mean(x),
  main = "",
  xlab= "",
  sub = "",
  Legend = TRUE,
  testAdherence = FALSE,
  Nsim = 9999,
  digits = 3
){

  if(testAdherence){
    Pvalue <- round(testPoissonDistribution(x, Nsim = Nsim), digits = digits)
    sub <- paste("P-value for adherence to a Poisson distribution =", Pvalue)
  }

  br <- seq(from=min(x)-1, to=max(x)+1, by=0.5)
  hist(x, breaks=br, freq=T,main=main, sub=sub, ylab="", xlab=xlab, col="lightblue", xlim=c(min(x)-1,max(x)+1))
  points((min(x):(max(x)+1))+0.2, dpois(min(x):(max(x)+1), lambda=lambda)*length(x), type="h", lwd=5, col="red")
  if(Legend){
    legend("topright", legend=c("Observed", "Predicted"),
      col=c("lightblue", "red"), lwd=c(15, 5), bty="n")
  }
}
# ============================================================================ #

# ============================================================================ #

testPoissonDistribution <- function(x, lambda=mean(x), Nsim=9999){

  K <- max(x, na.rm=TRUE) + 1
  Expected <- Observed <- numeric(K)
  for(k in 0:K){
    Expected[k] <- sum( dpois(x=k-1, lambda=lambda) )
    Observed[k] <- sum( x==k-1 )
  }

  Pvalue <- SimPvalues (Expected, Observed, Nsim=Nsim)
  return(Pvalue)
}

# ============================================================================ #